using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using UnityEngine;

namespace CheesyFX
{
    public class CollisionManager : MVRScript
    {
        private List<Atom> ignored = new List<Atom>();
        private JSONStorableString ignoredInfo = new JSONStorableString("ignoreInfo", "");

        private JSONStorableStringChooser atomChooser =
            new JSONStorableStringChooser("atomChooser", null, "", "Select Atom");
        
        public override void Init()
        {
            SyncChoices(SuperController.singleton.GetAtomUIDs());

            var button = CreateButton("Ignore Collision");
            button.button.onClick.AddListener(() => SetIgnore(true));
            button = CreateButton("Enable Collision");
            button.button.onClick.AddListener(() => SetIgnore(false));
            button = CreateButton("Reset");
            button.button.onClick.AddListener(Reset);
            
            CreateFilterablePopup(atomChooser, true);
            CreateTextField(ignoredInfo, true);
            
            RegisterString(ignoredInfo);
            
            SuperController.singleton.onAtomAddedHandlers += AddChoice;
            SuperController.singleton.onAtomUIDsChangedHandlers += SyncChoices;
        }

        public void OnDestroy()
        {
            SuperController.singleton.onAtomAddedHandlers -= AddChoice;
            SuperController.singleton.onAtomUIDsChangedHandlers -= SyncChoices;
            Reset();
        }

        private void AddChoice(Atom atom)
        {
            if (atomChooser.choices.Contains(atom.uid)) return;
            var choices = atomChooser.choices;
            choices.Insert(0, atom.uid);
            atomChooser.choices = new List<string>(choices);
        }

        private void SyncChoices(List<string> uids)
        {
            uids.Remove(containingAtom.uid);
            atomChooser.choices = new List<string>(uids);
        }

        private void SetIgnore(bool val)
        {
            SetIgnore(atomChooser.val, val);
        }
        
        private void SetIgnore(string uid, bool val)
        {
            var atom = SuperController.singleton.GetAtomByUid(uid);
            if (atom == null) return;
            if (ignored.Contains(atom))
            {
                if(val) return;
            }
            else if (!val) return;
            
            var myColliders = containingAtom.GetComponentsInChildren<Collider>();
            var hisColliders = atom.GetComponentsInChildren<Collider>();
            for (int i = 0; i < myColliders.Length; i++)
            {
                for (int j = 0; j < hisColliders.Length; j++)
                {
                    Physics.IgnoreCollision(myColliders[i], hisColliders[j], val);
                }
            }
            
            if(val) ignored.Add(atom);
            else ignored.Remove(atom);
            
            SyncInfo();
        }

        private void SyncInfo()
        {
            var ignoredUids = ignored.Select(x => x.uid).ToArray();
            ignoredInfo.val = string.Join("\n", ignoredUids);
        }

        private void Reset()
        {
            var ignoring = new List<Atom>(ignored);
            foreach (Atom atom in ignoring) SetIgnore(atom.uid, false);
            ignoredInfo.val = "";
        }
        
        public override void LateRestoreFromJSON(
            JSONClass jc,
            bool restorePhysical,
            bool restoreAppearance,
            bool setMissingToDefault
        )
        {
            if (jc.HasKey("ignoreInfo"))
            {
                var uidsToIgnore = ((string)jc["ignoreInfo"]).Split(new [] {"\n"}, StringSplitOptions.None);
                foreach (var uid in uidsToIgnore)
                {
                    SetIgnore(uid, true);
                }
            }
        }
    }
}